@@ -1,6 +1,7 @@
 /* events.c
 
 Copyright (C) 1999-2003 Tom Gilbert.
+Copyright (C) 2010-2011 Daniel Friesel.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -69,78 +70,61 @@ static void feh_event_handle_ButtonPress(XEvent * ev)
 	}
 
 	winwid = winwidget_get_from_window(ev->xbutton.window);
-	if (winwid && winwid->caption_entry) {
+	if (winwid == NULL || winwid->caption_entry) {
 		return;
 	}
 
 	if (!opt.no_menus && EV_IS_MENU_BUTTON(ev)) {
-		D(3, ("Menu Button Press event\n"));
-		if (winwid != NULL) {
-			winwidget_show_menu(winwid);
-		}
+		D(("Menu Button Press event\n"));
+		winwidget_show_menu(winwid);
 	} else if ((ev->xbutton.button == opt.rotate_button)
+		   && (winwid->type != WIN_TYPE_THUMBNAIL)
 		   && ((opt.no_rotate_ctrl_mask)
 		       || (ev->xbutton.state & ControlMask))) {
-		if (winwid != NULL) {
-			opt.mode = MODE_ROTATE;
-			winwid->mode = MODE_ROTATE;
-			D(3, ("rotate starting at %d, %d\n", ev->xbutton.x, ev->xbutton.y));
-		}
+		opt.mode = MODE_ROTATE;
+		winwid->mode = MODE_ROTATE;
+		D(("rotate starting at %d, %d\n", ev->xbutton.x, ev->xbutton.y));
 	} else if ((ev->xbutton.button == opt.blur_button)
+		   && (winwid->type != WIN_TYPE_THUMBNAIL)
 		   && ((opt.no_blur_ctrl_mask)
 		       || (ev->xbutton.state & ControlMask))) {
-		if (winwid != NULL) {
-			opt.mode = MODE_BLUR;
-			winwid->mode = MODE_BLUR;
-			D(3, ("blur starting at %d, %d\n", ev->xbutton.x, ev->xbutton.y));
-		}
+		opt.mode = MODE_BLUR;
+		winwid->mode = MODE_BLUR;
+		D(("blur starting at %d, %d\n", ev->xbutton.x, ev->xbutton.y));
 	} else if (ev->xbutton.button == opt.pan_button) {
-		D(3, ("Pan Button Press event\n"));
-		if (winwid != NULL) {
-			D(3, ("Next button, but could be pan mode\n"));
-			opt.mode = MODE_NEXT;
-			winwid->mode = MODE_NEXT;
-			D(3, ("click offset is %d,%d\n", ev->xbutton.x, ev->xbutton.y));
-			winwid->click_offset_x = ev->xbutton.x - winwid->im_x;
-			winwid->click_offset_y = ev->xbutton.y - winwid->im_y;
-		}
+		D(("Next button, but could be pan mode\n"));
+		opt.mode = MODE_NEXT;
+		winwid->mode = MODE_NEXT;
+		D(("click offset is %d,%d\n", ev->xbutton.x, ev->xbutton.y));
+		winwid->click_offset_x = ev->xbutton.x - winwid->im_x;
+		winwid->click_offset_y = ev->xbutton.y - winwid->im_y;
 	} else if (ev->xbutton.button == opt.zoom_button) {
-		D(3, ("Zoom Button Press event\n"));
-		if (winwid != NULL) {
-			D(3, ("Zoom mode baby!\n"));
-			opt.mode = MODE_ZOOM;
-			winwid->mode = MODE_ZOOM;
-			D(3, ("click offset is %d,%d\n", ev->xbutton.x, ev->xbutton.y));
-			winwid->click_offset_x = ev->xbutton.x;
-			winwid->click_offset_y = ev->xbutton.y;
-			winwid->old_zoom = winwid->zoom;
-			winwid->zoom = 1.0;
+		D(("Zoom Button Press event\n"));
+		opt.mode = MODE_ZOOM;
+		winwid->mode = MODE_ZOOM;
+		D(("click offset is %d,%d\n", ev->xbutton.x, ev->xbutton.y));
+		winwid->click_offset_x = ev->xbutton.x;
+		winwid->click_offset_y = ev->xbutton.y;
+		winwid->old_zoom = winwid->zoom;
 
-			/* required to adjust the image position in zoom mode */
-			winwid->im_click_offset_x = (winwid->click_offset_x
-					- winwid->im_x) / winwid->old_zoom;
-			winwid->im_click_offset_y = (winwid->click_offset_y
-					- winwid->im_y) / winwid->old_zoom;
-
-			winwidget_center_image(winwid);
-			winwidget_render_image(winwid, 0, 0);
-		}
+		/* required to adjust the image position in zoom mode */
+		winwid->im_click_offset_x = (winwid->click_offset_x
+				- winwid->im_x) / winwid->old_zoom;
+		winwid->im_click_offset_y = (winwid->click_offset_y
+				- winwid->im_y) / winwid->old_zoom;
 	} else if (ev->xbutton.button == opt.reload_button) {
-		D(3, ("Reload Button Press event\n"));
-		if (winwid != NULL)
-			feh_reload_image(winwid, 0, 0);
+		D(("Reload Button Press event\n"));
+			feh_reload_image(winwid, 0, 1);
 	} else if (ev->xbutton.button == opt.prev_button) {
-		D(3, ("Prev Button Press event\n"));
-		if ((winwid != NULL)
-				&& (winwid->type == WIN_TYPE_SLIDESHOW))
-			slideshow_change_image(winwid, SLIDE_PREV);
+		D(("Prev Button Press event\n"));
+		if (winwid->type == WIN_TYPE_SLIDESHOW)
+			slideshow_change_image(winwid, SLIDE_PREV, 1);
 	} else if (ev->xbutton.button == opt.next_button) {
-		D(3, ("Next Button Press event\n"));
-		if ((winwid != NULL)
-				&& (winwid->type == WIN_TYPE_SLIDESHOW))
-			slideshow_change_image(winwid, SLIDE_NEXT);
+		D(("Next Button Press event\n"));
+		if (winwid->type == WIN_TYPE_SLIDESHOW)
+			slideshow_change_image(winwid, SLIDE_NEXT, 1);
 	} else {
-		D(3, ("Received other ButtonPress event\n"));
+		D(("Received other ButtonPress event\n"));
 	}
 	return;
 }
@@ -168,39 +152,25 @@ static void feh_event_handle_ButtonRelease(XEvent * ev)
 	}
 
 	winwid = winwidget_get_from_window(ev->xbutton.window);
-	if (winwid && winwid->caption_entry) {
+	if (winwid == NULL || winwid->caption_entry) {
 		return;
 	}
 
-	if ((ev->xbutton.button == opt.menu_button)
-			&& (((!opt.menu_ctrl_mask)
-					&& ((!(ev->xbutton.state & ControlMask))
-						|| ((ev->xbutton.state & ControlMask)
-							&& (opt.menu_ctrl_mask)))))
-			&& (opt.no_menus))
-		winwidget_destroy_all();
-	else if (ev->xbutton.button == opt.pan_button) {
+	if (ev->xbutton.button == opt.pan_button) {
 		if (opt.mode == MODE_PAN) {
-			if (winwid != NULL) {
-				D(3, ("Disabling pan mode\n"));
-				opt.mode = MODE_NORMAL;
-				winwid->mode = MODE_NORMAL;
-				winwidget_sanitise_offsets(winwid);
-				winwidget_render_image(winwid, 0, 1);
-			}
+			D(("Disabling pan mode\n"));
+			opt.mode = MODE_NORMAL;
+			winwid->mode = MODE_NORMAL;
+			winwidget_sanitise_offsets(winwid);
+			winwidget_render_image(winwid, 0, 0);
 		} else if (opt.mode == MODE_NEXT) {
 			opt.mode = MODE_NORMAL;
-			if (winwid != NULL)
-				winwid->mode = MODE_NORMAL;
-			if ((winwid != NULL)
-					&& (winwid->type == WIN_TYPE_SLIDESHOW)) {
-				slideshow_change_image(winwid, SLIDE_NEXT);
-			} else if ((winwid != NULL)
-					&& (winwid->type == WIN_TYPE_THUMBNAIL)) {
+			winwid->mode = MODE_NORMAL;
+			if (winwid->type == WIN_TYPE_SLIDESHOW)
+				slideshow_change_image(winwid, SLIDE_NEXT, 1);
+			else if (winwid->type == WIN_TYPE_THUMBNAIL) {
 				feh_file *thumbfile;
-				winwidget thumbwin = NULL;
 				int x, y;
-				char *s;
 
 				x = ev->xbutton.x;
 				y = ev->xbutton.y;
@@ -209,51 +179,34 @@ static void feh_event_handle_ButtonRelease(XEvent * ev)
 				x /= winwid->zoom;
 				y /= winwid->zoom;
 				thumbfile = feh_thumbnail_get_file_from_coords(x, y);
-				if (thumbfile) {
-					if (!opt.thumb_title)
-						s = thumbfile->name;
-					else
-						s = feh_printf(opt.thumb_title, thumbfile);
-					thumbwin = winwidget_get_first_window_of_type(WIN_TYPE_THUMBNAIL_VIEWER);
-					if (!thumbwin) {
-						thumbwin = winwidget_create_from_file(
-								gib_list_add_front(NULL, thumbfile),
-								s, WIN_TYPE_THUMBNAIL_VIEWER);
-						if (thumbwin)
-							winwidget_show(thumbwin);
-					} else if (FEH_FILE(thumbwin->file->data) != thumbfile) {
-						free(thumbwin->file);
-						thumbwin->file = gib_list_add_front(NULL, thumbfile);
-						winwidget_rename(thumbwin, s);
-						feh_reload_image(thumbwin, 1, 0);
-					}
-				}
+				if (thumbfile)
+					feh_thumbnail_show_fullsize(thumbfile);
 			}
 		} else {
-			if (winwid != NULL) {
-				opt.mode = MODE_NORMAL;
-				winwid->mode = MODE_NORMAL;
-			}
+			opt.mode = MODE_NORMAL;
+			winwid->mode = MODE_NORMAL;
 		}
 	} else if ((ev->xbutton.button == opt.rotate_button)
 			|| (ev->xbutton.button == opt.zoom_button)) {
-		D(3, ("Mode-based Button Release event\n"));
-		if (winwid != NULL) {
-			D(3, ("Disabling mode\n"));
-			opt.mode = MODE_NORMAL;
-			winwid->mode = MODE_NORMAL;
+		D(("Disabling mode\n"));
+		opt.mode = MODE_NORMAL;
+		winwid->mode = MODE_NORMAL;
+
+		if ((ev->xbutton.button == opt.zoom_button)
+				&& (ev->xbutton.x == winwid->click_offset_x)
+				&& (ev->xbutton.y == winwid->click_offset_y)) {
+			winwid->zoom = 1.0;
+			winwidget_center_image(winwid);
+		} else
 			winwidget_sanitise_offsets(winwid);
-			winwidget_render_image(winwid, 0, 1);
-		}
+
+		winwidget_render_image(winwid, 0, 0);
 	} else if ((ev->xbutton.button == opt.blur_button)
 			&& ((opt.no_blur_ctrl_mask)
 				|| (ev->xbutton.state & ControlMask))) {
-		D(3, ("Blur Button Release event\n"));
-		if (winwid != NULL) {
-			D(3, ("Disabling Blur mode\n"));
-			opt.mode = MODE_NORMAL;
-			winwid->mode = MODE_NORMAL;
-		}
+		D(("Disabling Blur mode\n"));
+		opt.mode = MODE_NORMAL;
+		winwid->mode = MODE_NORMAL;
 	}
 	return;
 }
@@ -265,10 +218,10 @@ static void feh_event_handle_ConfigureNotify(XEvent * ev)
 		winwidget w = winwidget_get_from_window(ev->xconfigure.window);
 
 		if (w) {
-			D(3, ("configure size %dx%d\n", ev->xconfigure.width, ev->xconfigure.height));
+			D(("configure size %dx%d\n", ev->xconfigure.width, ev->xconfigure.height));
 			if ((w->w != ev->xconfigure.width)
 					|| (w->h != ev->xconfigure.height)) {
-				D(3, ("assigning size and rerendering\n"));
+				D(("assigning size and rerendering\n"));
 				w->w = ev->xconfigure.width;
 				w->h = ev->xconfigure.height;
 				w->had_resize = 1;
@@ -276,7 +229,7 @@ static void feh_event_handle_ConfigureNotify(XEvent * ev)
 					opt.geom_w = w->w;
 					opt.geom_h = w->h;
 				}
-				winwidget_render_image(w, 0, 1);
+				winwidget_render_image(w, 0, 0);
 			}
 		}
 	}
@@ -289,10 +242,10 @@ static void feh_event_handle_LeaveNotify(XEvent * ev)
 	if ((menu_root) && (ev->xcrossing.window == menu_root->win)) {
 		feh_menu_item *ii;
 
-		D(4, ("It is for a menu\n"));
+		D(("It is for a menu\n"));
 		for (ii = menu_root->items; ii; ii = ii->next) {
 			if (MENU_ITEM_IS_SELECTED(ii)) {
-				D(4, ("Unselecting menu\n"));
+				D(("Unselecting menu\n"));
 				MENU_ITEM_SET_NORMAL(ii);
 				menu_root->updates =
 					imlib_update_append_rect(menu_root->updates, ii->x, ii->y, ii->w, ii->h);
@@ -324,7 +277,7 @@ static void feh_event_handle_MotionNotify(XEvent * ev)
 		feh_menu *m;
 		feh_menu_item *selected_item, *mouseover_item;
 
-		D(3, ("motion notify with menus open\n"));
+		D(("motion notify with menus open\n"));
 		while (XCheckTypedWindowEvent(disp, ev->xmotion.window, MotionNotify, ev));
 
 		if (ev->xmotion.window == menu_cover) {
@@ -334,7 +287,7 @@ static void feh_event_handle_MotionNotify(XEvent * ev)
 			mouseover_item = feh_menu_find_at_xy(m, ev->xmotion.x, ev->xmotion.y);
 
 			if (selected_item != mouseover_item) {
-				D(4, ("selecting a menu item\n"));
+				D(("selecting a menu item\n"));
 				if (selected_item)
 					feh_menu_deselect_selected(m);
 				if ((mouseover_item)
@@ -399,7 +352,7 @@ static void feh_event_handle_MotionNotify(XEvent * ev)
 			winwid->im_y = winwid->click_offset_y
 					- (winwid->im_click_offset_y * winwid->zoom);
 
-			winwidget_render_image(winwid, 0, 0);
+			winwidget_render_image(winwid, 0, 1);
 		}
 	} else if ((opt.mode == MODE_PAN) || (opt.mode == MODE_NEXT)) {
 		int orig_x, orig_y;
@@ -411,7 +364,7 @@ static void feh_event_handle_MotionNotify(XEvent * ev)
 				opt.mode = MODE_PAN;
 				winwid->mode = MODE_PAN;
 			}
-			D(5, ("Panning\n"));
+			D(("Panning\n"));
 			orig_x = winwid->im_x;
 			orig_y = winwid->im_y;
 
@@ -420,7 +373,7 @@ static void feh_event_handle_MotionNotify(XEvent * ev)
 
 			winwidget_sanitise_offsets(winwid);
 
-			D(4, ("im_x %d, im_w %d, off %d, mx %d\n", winwid->im_x,
+			D(("im_x %d, im_w %d, off %d, mx %d\n", winwid->im_x,
 				winwid->im_w, winwid->click_offset_x, ev->xmotion.x));
 
 			/* XWarpPointer generates a MotionNotify event which we will
@@ -458,13 +411,13 @@ static void feh_event_handle_MotionNotify(XEvent * ev)
 
 			if ((winwid->im_x != orig_x)
 					|| (winwid->im_y != orig_y))
-				winwidget_render_image(winwid, 0, 0);
+				winwidget_render_image(winwid, 0, 1);
 		}
 	} else if (opt.mode == MODE_ROTATE) {
 		while (XCheckTypedWindowEvent(disp, ev->xmotion.window, MotionNotify, ev));
 		winwid = winwidget_get_from_window(ev->xmotion.window);
 		if (winwid) {
-			D(5, ("Rotating\n"));
+			D(("Rotating\n"));
 			if (!winwid->has_rotated) {
 				Imlib_Image temp;
 
@@ -477,8 +430,8 @@ static void feh_event_handle_MotionNotify(XEvent * ev)
 				winwid->has_rotated = 1;
 			}
 			winwid->im_angle = (ev->xmotion.x - winwid->w / 2) / ((double) winwid->w / 2) * 3.1415926535;
-			D(5, ("angle: %f\n", winwid->im_angle));
-			winwidget_render_image(winwid, 0, 0);
+			D(("angle: %f\n", winwid->im_angle));
+			winwidget_render_image(winwid, 0, 1);
 		}
 	} else if (opt.mode == MODE_BLUR) {
 		while (XCheckTypedWindowEvent(disp, ev->xmotion.window, MotionNotify, ev));
@@ -487,75 +440,32 @@ static void feh_event_handle_MotionNotify(XEvent * ev)
 			Imlib_Image temp, ptr;
 			signed int blur_radius;
 
-			D(5, ("Blurring\n"));
+			D(("Blurring\n"));
 
 			temp = gib_imlib_clone_image(winwid->im);
 			blur_radius = (((double) ev->xmotion.x / winwid->w) * 20) - 10;
-			D(5, ("angle: %d\n", blur_radius));
+			D(("angle: %d\n", blur_radius));
 			if (blur_radius > 0)
 				gib_imlib_image_sharpen(temp, blur_radius);
 			else
 				gib_imlib_image_blur(temp, 0 - blur_radius);
 			ptr = winwid->im;
 			winwid->im = temp;
-			winwidget_render_image(winwid, 0, 0);
+			winwidget_render_image(winwid, 0, 1);
 			gib_imlib_free_image_and_decache(winwid->im);
 			winwid->im = ptr;
 		}
 	} else {
 		while (XCheckTypedWindowEvent(disp, ev->xmotion.window, MotionNotify, ev));
 		winwid = winwidget_get_from_window(ev->xmotion.window);
-		if (winwid != NULL) {
-			if (winwid->type == WIN_TYPE_ABOUT) {
-				Imlib_Image orig_im;
-				int x, y;
+		if ((winwid != NULL) && (winwid->type == WIN_TYPE_THUMBNAIL)) {
+			feh_thumbnail *thumbnail;
+			int x, y;
 
-				x = ev->xmotion.x - winwid->im_x;
-				y = ev->xmotion.y - winwid->im_y;
-				orig_im = winwid->im;
-				winwid->im = gib_imlib_clone_image(orig_im);
-				imlib_context_set_image(winwid->im);
-				imlib_apply_filter("bump_map_point(x=[],y=[],map="
-						PREFIX "/share/feh/images/about.png);", &x, &y);
-				winwidget_render_image(winwid, 0, 1);
-				gib_imlib_free_image_and_decache(winwid->im);
-				winwid->im = orig_im;
-			} else if (winwid->type == WIN_TYPE_THUMBNAIL) {
-				static feh_thumbnail *last_thumb = NULL;
-				feh_thumbnail *thumbnail;
-				int x, y;
-
-				x = (ev->xbutton.x - winwid->im_x) / winwid->zoom;
-				y = (ev->xbutton.y - winwid->im_y) / winwid->zoom;
-				thumbnail = feh_thumbnail_get_thumbnail_from_coords(x, y);
-				if (thumbnail != last_thumb) {
-					if (thumbnail) {
-						Imlib_Image origwin;
-
-						origwin = winwid->im;
-						winwid->im = gib_imlib_clone_image(origwin);
-						gib_imlib_image_fill_rectangle(winwid->im,
-								thumbnail->x, thumbnail->y, thumbnail->w,
-								thumbnail->h, 50, 50, 255, 100);
-						gib_imlib_image_draw_rectangle(winwid->im,
-								thumbnail->x, thumbnail->y, thumbnail->w,
-								thumbnail->h, 255, 255, 255, 255);
-						gib_imlib_image_draw_rectangle(winwid->im,
-								thumbnail->x + 1, thumbnail->y + 1,
-								thumbnail->w - 2, thumbnail->h - 2,
-								0, 0, 0, 255);
-						gib_imlib_image_draw_rectangle(winwid->im,
-								thumbnail->x + 2, thumbnail->y + 2,
-								thumbnail->w - 4, thumbnail->h - 4,
-								255, 255, 255, 255);
-						winwidget_render_image(winwid, 0, 1);
-						gib_imlib_free_image_and_decache(winwid->im);
-						winwid->im = origwin;
-					} else
-						winwidget_render_image(winwid, 0, 1);
-				}
-				last_thumb = thumbnail;
-			}
+			x = (ev->xbutton.x - winwid->im_x) / winwid->zoom;
+			y = (ev->xbutton.y - winwid->im_y) / winwid->zoom;
+			thumbnail = feh_thumbnail_get_thumbnail_from_coords(x, y);
+			feh_thumbnail_select(winwid, thumbnail);
 		}
 	}
 	return;
