@@ -319,30 +319,53 @@ char *feh_http_load_image(char *url)
 
 #endif				/* HAVE_LIBCURL */
 
+void feh_imlib_image_fill_text_bg(Imlib_Image im, int w, int h)
+{
+	gib_imlib_image_set_has_alpha(im, 1);
+
+	imlib_context_set_blend(0);
+
+	if (opt.text_bg == TEXT_BG_CLEAR)
+		gib_imlib_image_fill_rectangle(im, 0, 0, w, h, 0, 0, 0, 0);
+	else
+		gib_imlib_image_fill_rectangle(im, 0, 0, w, h, 0, 0, 0, 127);
+
+	imlib_context_set_blend(1);
+}
+
+static Imlib_Font feh_load_font(winwidget w)
+{
+	static Imlib_Font fn = NULL;
+
+	if (opt.font)
+		fn = gib_imlib_load_font(opt.font);
+
+	if (!fn) {
+		if (w && w->full_screen)
+			fn = gib_imlib_load_font(DEFAULT_FONT_BIG);
+		else
+			fn = gib_imlib_load_font(DEFAULT_FONT);
+	}
+
+	if (!fn) {
+		eprintf("Couldn't load font to draw a message");
+	}
+
+	return fn;
+}
+
+
 void feh_draw_zoom(winwidget w)
 {
 	static Imlib_Font fn = NULL;
 	int tw = 0, th = 0;
 	Imlib_Image im = NULL;
 	char buf[100];
-	static DATA8 atab[256];
 
 	if (!w->im)
 		return;
 
-	if (opt.font)
-		fn = gib_imlib_load_font(opt.font);
-
-	if (!fn) {
-		fn = gib_imlib_load_font(DEFAULT_FONT);
-	}
-
-	if (!fn) {
-		weprintf("Couldn't load font for zoom printing");
-		return;
-	}
-
-	memset(atab, 0, sizeof(atab));
+	fn = feh_load_font(w);
 
 	snprintf(buf, sizeof(buf), "%.0f%%, %dx%d", w->zoom * 100,
 			(int) (w->im_w * w->zoom), (int) (w->im_h * w->zoom));
@@ -356,9 +379,7 @@ void feh_draw_zoom(winwidget w)
 	if (!im)
 		eprintf("Couldn't create image. Out of memory?");
 
-	gib_imlib_image_set_has_alpha(im, 1);
-	gib_imlib_apply_color_modifier_to_rectangle(im, 0, 0, tw, th, NULL, NULL, NULL, atab);
-	gib_imlib_image_fill_rectangle(im, 0, 0, tw, th, 0, 0, 0, 0);
+	feh_imlib_image_fill_text_bg(im, tw, th);
 
 	gib_imlib_text_draw(im, fn, NULL, 2, 2, buf, IMLIB_TEXT_TO_RIGHT, 0, 0, 0, 255);
 	gib_imlib_text_draw(im, fn, NULL, 1, 1, buf, IMLIB_TEXT_TO_RIGHT, 255, 255, 255, 255);
@@ -394,21 +415,11 @@ void feh_draw_errstr(winwidget w)
 	static Imlib_Font fn = NULL;
 	int tw = 0, th = 0;
 	Imlib_Image im = NULL;
-	static DATA8 atab[256];
 
 	if (!w->im)
 		return;
 
-	if (opt.font)
-		fn = gib_imlib_load_font(opt.font);
-
-	if (!fn)
-		fn = gib_imlib_load_font(DEFAULT_FONT);
-
-	if (!fn)
-		eprintf("Unable to draw error message. Dying to be safe.");
-
-	memset(atab, 0, sizeof(atab));
+	fn = feh_load_font(NULL);
 
 	/* Work out how high the font is */
 	gib_imlib_get_text_size(fn, w->errstr, NULL, &tw, &th, IMLIB_TEXT_TO_RIGHT);
@@ -419,9 +430,7 @@ void feh_draw_errstr(winwidget w)
 	if (!im)
 		eprintf("Couldn't create errstr image. Out of memory?");
 
-	gib_imlib_image_set_has_alpha(im, 1);
-	gib_imlib_apply_color_modifier_to_rectangle(im, 0, 0, tw, th, NULL, NULL, NULL, atab);
-	gib_imlib_image_fill_rectangle(im, 0, 0, tw, th, 0, 0, 0, 0);
+	feh_imlib_image_fill_text_bg(im, tw, th);
 
 	gib_imlib_text_draw(im, fn, NULL, 2, 2, w->errstr, IMLIB_TEXT_TO_RIGHT, 0, 0, 0, 255);
 	gib_imlib_text_draw(im, fn, NULL, 1, 1, w->errstr, IMLIB_TEXT_TO_RIGHT, 255, 0, 0, 255);
@@ -436,7 +445,6 @@ void feh_draw_filename(winwidget w)
 	static Imlib_Font fn = NULL;
 	int tw = 0, th = 0, nw = 0;
 	Imlib_Image im = NULL;
-	static DATA8 atab[256];
 	char *s = NULL;
 	int len = 0;
 
@@ -444,22 +452,7 @@ void feh_draw_filename(winwidget w)
 			|| (!FEH_FILE(w->file->data)->filename))
 		return;
 
-	if (opt.font)
-		fn = gib_imlib_load_font(opt.font);
-
-	if (!fn) {
-		if (w->full_screen)
-			fn = gib_imlib_load_font(DEFAULT_FONT_BIG);
-		else
-			fn = gib_imlib_load_font(DEFAULT_FONT);
-	}
-
-	if (!fn) {
-		eprintf("Couldn't load font for filename printing");
-		return;
-	}
-
-	memset(atab, 0, sizeof(atab));
+	fn = feh_load_font(w);
 
 	/* Work out how high the font is */
 	gib_imlib_get_text_size(fn, FEH_FILE(w->file->data)->filename, NULL, &tw,
@@ -484,10 +477,7 @@ void feh_draw_filename(winwidget w)
 	if (!im)
 		eprintf("Couldn't create image. Out of memory?");
 
-	gib_imlib_image_set_has_alpha(im, 1);
-	gib_imlib_apply_color_modifier_to_rectangle(im, 0, 0, tw, 2 * th, NULL,
-			NULL, NULL, atab);
-	gib_imlib_image_fill_rectangle(im, 0, 0, tw, 2 * th, 0, 0, 0, 0);
+	feh_imlib_image_fill_text_bg(im, tw, 2 * th);
 
 	gib_imlib_text_draw(im, fn, NULL, 2, 2, FEH_FILE(w->file->data)->filename,
 			IMLIB_TEXT_TO_RIGHT, 0, 0, 0, 255);
@@ -509,72 +499,69 @@ void feh_draw_filename(winwidget w)
 void feh_draw_info(winwidget w)
 {
 	static Imlib_Font fn = NULL;
-	int tw = 0, th = 0;
+	int width = 0, height = 0, line_width = 0, line_height = 0;
 	Imlib_Image im = NULL;
-	static DATA8 atab[256];
-	int no_lines = 0;
+	int no_lines = 0, i;
 	char *info_cmd;
-	char info_buf[256];
+	char info_line[256];
+	char *info_buf[128];
 	FILE *info_pipe;
 
 	if ((!w->file) || (!FEH_FILE(w->file->data))
 			|| (!FEH_FILE(w->file->data)->filename))
 		return;
 
-	if (opt.font)
-		fn = gib_imlib_load_font(opt.font);
-
-	if (!fn) {
-		if (w->full_screen)
-			fn = gib_imlib_load_font(DEFAULT_FONT_BIG);
-		else
-			fn = gib_imlib_load_font(DEFAULT_FONT);
-	}
-
-	if (!fn) {
-		eprintf("Couldn't load font for filename printing");
-		return;
-	}
+	fn = feh_load_font(w);
 
 	info_cmd = feh_printf(opt.info_cmd, FEH_FILE(w->file->data));
 
-	memset(atab, 0, sizeof(atab));
-
-	gib_imlib_get_text_size(fn, "w", NULL, &tw, &th, IMLIB_TEXT_TO_RIGHT);
-
 	info_pipe = popen(info_cmd, "r");
 
-	im = imlib_create_image(290 * tw, 20 * th);
-	if (!im)
-		eprintf("Couldn't create image. Out of memory?");
-
-	gib_imlib_image_set_has_alpha(im, 1);
-	gib_imlib_apply_color_modifier_to_rectangle(im, 0, 0, 290 * tw, 20 * th, NULL, NULL, NULL, atab);
-	gib_imlib_image_fill_rectangle(im, 0, 0, 290 * tw, 20 * th, 0, 0, 0, 0);
-
 	if (!info_pipe) {
-		gib_imlib_text_draw(im, fn, NULL, 2, 2,
-				"Error runnig info command", IMLIB_TEXT_TO_RIGHT,
-				255, 0, 0, 255);
-		gib_imlib_get_text_size(fn, "Error running info command", NULL, &tw, &th,
-				IMLIB_TEXT_TO_RIGHT);
+		info_buf[0] = estrdup("Failed to run info command");
+		gib_imlib_get_text_size(fn, info_buf[0], NULL, &width, &height, IMLIB_TEXT_TO_RIGHT);
 		no_lines = 1;
 	}
 	else {
-		while ((no_lines < 20) && fgets(info_buf, 256, info_pipe)) {
-			info_buf[strlen(info_buf)-1] = '\0';
-			gib_imlib_text_draw(im, fn, NULL, 2, (no_lines*th)+2, info_buf,
-					IMLIB_TEXT_TO_RIGHT, 0, 0, 0, 255);
-			gib_imlib_text_draw(im, fn, NULL, 1, (no_lines*th)+1, info_buf,
-					IMLIB_TEXT_TO_RIGHT, 255, 255, 255, 255);
+		while ((no_lines < 128) && fgets(info_line, 256, info_pipe)) {
+			if (info_line[strlen(info_line)-1] == '\n')
+				info_line[strlen(info_line)-1] = '\0';
+
+			gib_imlib_get_text_size(fn, info_line, NULL, &line_width,
+					&line_height, IMLIB_TEXT_TO_RIGHT);
+
+			if (line_height > height)
+				height = line_height;
+			if (line_width > width)
+				width = line_width;
+
+			info_buf[no_lines] = estrdup(info_line);
+
 			no_lines++;
 		}
 		pclose(info_pipe);
 	}
 
+	height *= no_lines;
+	width += 4;
 
-	gib_imlib_render_image_on_drawable(w->bg_pmap, im, 2,
-			w->h - (th * no_lines) - 2, 1, 1, 0);
+	im = imlib_create_image(width, height);
+	if (!im)
+		eprintf("Couldn't create image. Out of memory?");
+
+	feh_imlib_image_fill_text_bg(im, width, height);
+
+	for (i = 0; i < no_lines; i++) {
+		gib_imlib_text_draw(im, fn, NULL, 2, (i * line_height) + 2,
+				info_buf[i], IMLIB_TEXT_TO_RIGHT, 0, 0, 0, 255);
+		gib_imlib_text_draw(im, fn, NULL, 1, (i * line_height) + 1,
+				info_buf[i], IMLIB_TEXT_TO_RIGHT, 255, 255, 255, 255);
+
+		free(info_buf[i]);
+	}
+
+	gib_imlib_render_image_on_drawable(w->bg_pmap, im, 0,
+			w->h - height, 1, 1, 0);
 
 	gib_imlib_free_image_and_decache(im);
 	return;
@@ -620,7 +607,6 @@ void feh_draw_caption(winwidget w)
 	int tw = 0, th = 0, ww, hh;
 	int x, y;
 	Imlib_Image im = NULL;
-	static DATA8 atab[256];
 	char *p;
 	gib_list *lines, *l;
 	static gib_style *caption_style = NULL;
@@ -666,22 +652,7 @@ void feh_draw_caption(winwidget w)
 	caption_style->bits = gib_list_add_front(caption_style->bits,
 		gib_style_bit_new(1, 1, 0, 0, 0, 255));
 
-	if (opt.font)
-		fn = gib_imlib_load_font(opt.font);
-
-	if (!fn) {
-		if (w->full_screen)
-			fn = gib_imlib_load_font(DEFAULT_FONT_BIG);
-		else
-			fn = gib_imlib_load_font(DEFAULT_FONT);
-	}
-
-	if (!fn) {
-		eprintf("Couldn't load font for caption printing");
-		return;
-	}
-
-	memset(atab, 0, sizeof(atab));
+	fn = feh_load_font(w);
 
 	if (*(file->caption) == '\0') {
 		p = estrdup("Caption entry mode - Hit ESC to cancel");
@@ -716,9 +687,7 @@ void feh_draw_caption(winwidget w)
 	if (!im)
 		eprintf("Couldn't create image. Out of memory?");
 
-	gib_imlib_image_set_has_alpha(im, 1);
-	gib_imlib_apply_color_modifier_to_rectangle(im, 0, 0, tw, th, NULL, NULL, NULL, atab);
-	gib_imlib_image_fill_rectangle(im, 0, 0, tw, th, 0, 0, 0, 0);
+	feh_imlib_image_fill_text_bg(im, tw, th);
 
 	l = lines;
 	x = 0;
@@ -786,7 +755,7 @@ void feh_display_status(char stat)
 	return;
 }
 
-void feh_edit_inplace_orient(winwidget w, int orientation)
+void feh_edit_inplace(winwidget w, int op)
 {
 	int ret;
 	Imlib_Image old;
@@ -794,14 +763,21 @@ void feh_edit_inplace_orient(winwidget w, int orientation)
 		return;
 
 	if (!strcmp(gib_imlib_image_format(w->im), "jpeg")) {
-		feh_edit_inplace_lossless_rotate(w, orientation);
+		feh_edit_inplace_lossless(w, op);
 		feh_reload_image(w, 1, 1);
 		return;
 	}
 
 	ret = feh_load_image(&old, FEH_FILE(w->file->data));
 	if (ret) {
-		gib_imlib_image_orientate(old, orientation);
+		if (op == INPLACE_EDIT_FLIP) {
+			imlib_context_set_image(old);
+			imlib_image_flip_vertical();
+		} else if (op == INPLACE_EDIT_MIRROR) {
+			imlib_context_set_image(old);
+			imlib_image_flip_horizontal();
+		} else
+			gib_imlib_image_orientate(old, op);
 		gib_imlib_save_image(old, FEH_FILE(w->file->data)->filename);
 		gib_imlib_free_image(old);
 		feh_reload_image(w, 1, 1);
@@ -910,37 +886,47 @@ gib_list *feh_wrap_string(char *text, int wrap_width, Imlib_Font fn, gib_style *
 	return lines;
 }
 
-void feh_edit_inplace_lossless_rotate(winwidget w, int orientation)
+void feh_edit_inplace_lossless(winwidget w, int op)
 {
 	char *filename = FEH_FILE(w->file->data)->filename;
-	char rotate_str[4];
 	int len = strlen(filename) + 1;
 	char *file_str = emalloc(len);
-	int rotatearg = 90 * orientation;
 	int pid, status;
+	char op_name[]  = "rotate";     /* message */
+	char op_op[]    = "-rotate";    /* jpegtran option */
+	char op_value[] = "horizontal"; /* jpegtran option's value */
+
+	if (op == INPLACE_EDIT_FLIP) {
+		sprintf(op_name,  "flip");
+		sprintf(op_op,    "-flip");
+		sprintf(op_value, "vertical");
+	} else if (op == INPLACE_EDIT_MIRROR) {
+		sprintf(op_name,  "mirror");
+		sprintf(op_op,    "-flip");
+	} else
+		snprintf(op_value, 4, "%d", 90 * op);
 
-	snprintf(rotate_str, 4, "%d", rotatearg);
 	snprintf(file_str, len, "%s", filename);
 
 	if ((pid = fork()) < 0) {
-		im_weprintf(w, "lossless rotate: fork failed:");
+		im_weprintf(w, "lossless %s: fork failed:", op_name);
 		return;
 	} else if (pid == 0) {
 
-		execlp("jpegtran", "jpegtran", "-copy", "all", "-rotate",
-				rotate_str, "-outfile", file_str, file_str, NULL);
+		execlp("jpegtran", "jpegtran", "-copy", "all", op_op, op_value,
+				"-outfile", file_str, file_str, NULL);
 
-		im_weprintf(w, "lossless rotate: Is 'jpegtran' installed? Failed to exec:");
+		im_weprintf(w, "lossless %s: Is 'jpegtran' installed? Failed to exec:", op_name);
 		return;
 	} else {
 		waitpid(pid, &status, 0);
 
 		if (!WIFEXITED(status) || WEXITSTATUS(status) != 0) {
 			im_weprintf(w,
-					"lossless rotate: Got exitcode %d from jpegtran."
+					"lossless %s: Got exitcode %d from jpegtran."
 					" Commandline was: "
-					"jpegtran -copy all -rotate %s -outfile %s %s",
-					status >> 8, rotate_str, file_str, file_str);
+					"jpegtran -copy all %s %s -outfile %s %s",
+					op_name, status >> 8, op_op, op_value, file_str, file_str);
 			return;
 		}
 	}
@@ -954,7 +940,6 @@ void feh_draw_actions(winwidget w)
 	int max_tw = 0;
 	int line_th = 0;
 	Imlib_Image im = NULL;
-	static DATA8 atab[256];
 	int i = 0;
 	int num_actions = 0;
 	int cur_action = 0;
@@ -977,22 +962,7 @@ void feh_draw_actions(winwidget w)
 			|| (!FEH_FILE(w->file->data)->filename))
 		return;
 
-	if (opt.font)
-		fn = gib_imlib_load_font(opt.font);
-
-	if (!fn) {
-		if (w->full_screen)
-			fn = gib_imlib_load_font(DEFAULT_FONT_BIG);
-		else
-			fn = gib_imlib_load_font(DEFAULT_FONT);
-	}
-
-	if (!fn) {
-		eprintf("Couldn't load font for actions printing");
-		return;
-	}
-
-	memset(atab, 0, sizeof(atab));
+	fn = feh_load_font(w);
 
 	gib_imlib_get_text_size(fn, "defined actions:", NULL, &tw, &th, IMLIB_TEXT_TO_RIGHT);
 /* Check for the widest line */
@@ -1026,9 +996,7 @@ void feh_draw_actions(winwidget w)
 	if (!im)
 		eprintf("Couldn't create image. Out of memory?");
 
-	gib_imlib_image_set_has_alpha(im, 1);
-	gib_imlib_apply_color_modifier_to_rectangle(im, 0, 0, tw, th, NULL, NULL, NULL, atab);
-	gib_imlib_image_fill_rectangle(im, 0, 0, tw, th, 0, 0, 0, 0);
+	feh_imlib_image_fill_text_bg(im, tw, th);
 
 	gib_imlib_text_draw(im, fn, NULL, 2, 2, "defined actions:", IMLIB_TEXT_TO_RIGHT, 0, 0, 0, 255);
 	gib_imlib_text_draw(im, fn, NULL, 1, 1, "defined actions:", IMLIB_TEXT_TO_RIGHT, 255, 255, 255, 255);
