@@ -50,25 +50,30 @@ static void feh_set_parse_kb_partial(fehkey *key, int index, char *ks) {
 		return;
 	}
 
-	if (ks[1] == '-') {
-		switch (ks[0]) {
+	while (cur[1] == '-') {
+		switch (cur[0]) {
 			case 'C':
-				mod = ControlMask;
+				mod |= ControlMask;
+				break;
+			case 'S':
+				mod |= ShiftMask;
 				break;
 			case '1':
-				mod = Mod1Mask;
+				mod |= Mod1Mask;
 				break;
 			case '4':
-				mod = Mod4Mask;
+				mod |= Mod4Mask;
 				break;
 			default:
-				weprintf("keys: invalid modifier %c in %s", ks[0], ks);
+				weprintf("keys: invalid modifier %c in \"%s\"", cur[0], ks);
 				break;
 		}
-		cur = ks + 2;
+		cur += 2;
 	}
 
 	key->keysyms[index] = XStringToKeysym(cur);
+	if (isascii(key->keysyms[index]))
+		mod &= ~ShiftMask;
 	key->keystates[index] = mod;
 
 	if (key->keysyms[index] == NoSymbol)
@@ -97,6 +102,10 @@ void init_keyevents(void) {
 	feh_set_kb(&keys.scroll_right,0,XK_KP_Right  , 4, XK_Right     , 0, 0);
 	feh_set_kb(&keys.scroll_down,0, XK_KP_Down   , 4, XK_Down      , 0, 0);
 	feh_set_kb(&keys.scroll_up , 0, XK_KP_Up     , 4, XK_Up        , 0, 0);
+	feh_set_kb(&keys.scroll_left_page , 8, XK_Left , 0, 0          , 0, 0);
+	feh_set_kb(&keys.scroll_right_page, 8, XK_Right, 0, 0          , 0, 0);
+	feh_set_kb(&keys.scroll_down_page , 8, XK_Down , 0, 0          , 0, 0);
+	feh_set_kb(&keys.scroll_up_page   , 8, XK_Up   , 0, 0          , 0, 0);
 	feh_set_kb(&keys.prev_img  , 0, XK_Left      , 0, XK_p         , 0, XK_BackSpace);
 	feh_set_kb(&keys.next_img  , 0, XK_Right     , 0, XK_n         , 0, XK_space);
 	feh_set_kb(&keys.jump_back , 0, XK_Page_Up   , 0, XK_KP_Page_Up, 0, 0);
@@ -138,6 +147,8 @@ void init_keyevents(void) {
 	feh_set_kb(&keys.save_filelist, 0, XK_f, 0, 0, 0, 0);
 	feh_set_kb(&keys.orient_1, 0, XK_greater, 0, 0, 0, 0);
 	feh_set_kb(&keys.orient_3, 0, XK_less, 0, 0, 0, 0);
+	feh_set_kb(&keys.flip, 0, XK_underscore, 0, 0, 0, 0);
+	feh_set_kb(&keys.mirror, 0, XK_bar, 0, 0, 0, 0);
 	feh_set_kb(&keys.reload_minus, 0, XK_minus, 0, 0, 0, 0);
 	feh_set_kb(&keys.reload_plus, 0, XK_plus, 0, 0, 0, 0);
 
@@ -191,6 +202,14 @@ void init_keyevents(void) {
 			cur_kb = &keys.scroll_up;
 		else if (!strcmp(action, "scroll_down"))
 			cur_kb = &keys.scroll_down;
+		else if (!strcmp(action, "scroll_right_page"))
+			cur_kb = &keys.scroll_right_page;
+		else if (!strcmp(action, "scroll_left_page"))
+			cur_kb = &keys.scroll_left_page;
+		else if (!strcmp(action, "scroll_up_page"))
+			cur_kb = &keys.scroll_up_page;
+		else if (!strcmp(action, "scroll_down_page"))
+			cur_kb = &keys.scroll_down_page;
 		else if (!strcmp(action, "prev_img"))
 			cur_kb = &keys.prev_img;
 		else if (!strcmp(action, "next_img"))
@@ -273,6 +292,10 @@ void init_keyevents(void) {
 			cur_kb = &keys.orient_1;
 		else if (!strcmp(action, "orient_3"))
 			cur_kb = &keys.orient_3;
+		else if (!strcmp(action, "flip"))
+			cur_kb = &keys.flip;
+		else if (!strcmp(action, "mirror"))
+			cur_kb = &keys.mirror;
 		else if (!strcmp(action, "reload_minus"))
 			cur_kb = &keys.reload_minus;
 		else if (!strcmp(action, "reload_plus"))
@@ -308,15 +331,20 @@ void feh_event_invoke_action(winwidget winwid, unsigned char action)
 	if (opt.actions[action]) {
 		if (opt.slideshow) {
 			feh_action_run(FEH_FILE(winwid->file->data), opt.actions[action]);
-			winwidget_update_caption(winwid);
 
-			if (! opt.hold_actions[action])
-				slideshow_change_image(winwid, SLIDE_NEXT);
+			if (opt.hold_actions[action])
+				feh_reload_image(winwid, 1, 1);
+			else
+				slideshow_change_image(winwid, SLIDE_NEXT, 1);
 
 		} else if ((winwid->type == WIN_TYPE_SINGLE)
 				|| (winwid->type == WIN_TYPE_THUMBNAIL_VIEWER)) {
 			feh_action_run(FEH_FILE(winwid->file->data), opt.actions[action]);
-			winwidget_destroy(winwid);
+
+			if (opt.hold_actions[action])
+				feh_reload_image(winwid, 1, 1);
+			else
+				winwidget_destroy(winwid);
 		} else if (winwid->type == WIN_TYPE_THUMBNAIL)
 			fputs("actions from the main thumb window aren't currently supported!\n"
 					"For now, open the image to perform the action on it.\n",
@@ -327,7 +355,6 @@ void feh_event_invoke_action(winwidget winwid, unsigned char action)
 
 void feh_event_handle_keypress(XEvent * ev)
 {
-	int len;
 	int state;
 	char kbuf[20];
 	KeySym keysym;
@@ -345,8 +372,11 @@ void feh_event_handle_keypress(XEvent * ev)
 	}
 
 	kev = (XKeyEvent *) ev;
-	len = XLookupString(&ev->xkey, (char *) kbuf, sizeof(kbuf), &keysym, NULL);
-	state = kev->state & (ControlMask | Mod1Mask | Mod4Mask);
+	XLookupString(&ev->xkey, (char *) kbuf, sizeof(kbuf), &keysym, NULL);
+	state = kev->state & (ControlMask | ShiftMask | Mod1Mask | Mod4Mask);
+
+	if (isascii(keysym))
+		state &= ~ShiftMask;
 
 	/* menus are showing, so this is a menu control keypress */
 	if (ev->xbutton.window == menu_cover) {
@@ -423,13 +453,13 @@ void feh_event_handle_keypress(XEvent * ev)
 
 	if (feh_is_kp(&keys.next_img, keysym, state)) {
 		if (opt.slideshow)
-			slideshow_change_image(winwid, SLIDE_NEXT);
+			slideshow_change_image(winwid, SLIDE_NEXT, 1);
 		else if (winwid->type == WIN_TYPE_THUMBNAIL)
 			feh_thumbnail_select_next(winwid, 1);
 	}
 	else if (feh_is_kp(&keys.prev_img, keysym, state)) {
 		if (opt.slideshow)
-			slideshow_change_image(winwid, SLIDE_PREV);
+			slideshow_change_image(winwid, SLIDE_PREV, 1);
 		else if (winwid->type == WIN_TYPE_THUMBNAIL)
 			feh_thumbnail_select_prev(winwid, 1);
 	}
@@ -449,15 +479,31 @@ void feh_event_handle_keypress(XEvent * ev)
 		winwid->im_y += 20;
 		winwidget_render_image(winwid, 0, 1);
 	}
+	else if (feh_is_kp(&keys.scroll_right_page, keysym, state)) {
+		winwid->im_x -= winwid->w;
+		winwidget_render_image(winwid, 0, 0);
+	}
+	else if (feh_is_kp(&keys.scroll_left_page, keysym, state)) {
+		winwid->im_x += winwid->w;
+		winwidget_render_image(winwid, 0, 0);
+	}
+	else if (feh_is_kp(&keys.scroll_down_page, keysym, state)) {
+		winwid->im_y -= winwid->h;
+		winwidget_render_image(winwid, 0, 0);
+	}
+	else if (feh_is_kp(&keys.scroll_up_page, keysym, state)) {
+		winwid->im_y += winwid->h;
+		winwidget_render_image(winwid, 0, 0);
+	}
 	else if (feh_is_kp(&keys.jump_back, keysym, state)) {
 		if (opt.slideshow)
-			slideshow_change_image(winwid, SLIDE_JUMP_BACK);
+			slideshow_change_image(winwid, SLIDE_JUMP_BACK, 1);
 		else if (winwid->type == WIN_TYPE_THUMBNAIL)
 			feh_thumbnail_select_prev(winwid, 10);
 	}
 	else if (feh_is_kp(&keys.jump_fwd, keysym, state)) {
 		if (opt.slideshow)
-			slideshow_change_image(winwid, SLIDE_JUMP_FWD);
+			slideshow_change_image(winwid, SLIDE_JUMP_FWD, 1);
 		else if (winwid->type == WIN_TYPE_THUMBNAIL)
 			feh_thumbnail_select_next(winwid, 10);
 	}
@@ -476,11 +522,11 @@ void feh_event_handle_keypress(XEvent * ev)
 	}
 	else if (feh_is_kp(&keys.jump_first, keysym, state)) {
 		if (opt.slideshow)
-			slideshow_change_image(winwid, SLIDE_FIRST);
+			slideshow_change_image(winwid, SLIDE_FIRST, 1);
 	}
 	else if (feh_is_kp(&keys.jump_last, keysym, state)) {
 		if (opt.slideshow)
-			slideshow_change_image(winwid, SLIDE_LAST);
+			slideshow_change_image(winwid, SLIDE_LAST, 1);
 	}
 	else if (feh_is_kp(&keys.action_0, keysym, state)) {
 		feh_event_invoke_action(winwid, 0);
@@ -576,7 +622,7 @@ void feh_event_handle_keypress(XEvent * ev)
 		opt.hide_pointer = !opt.hide_pointer;
 	}
 	else if (feh_is_kp(&keys.jump_random, keysym, state)) {
-		slideshow_change_image(winwid, SLIDE_RAND);
+		slideshow_change_image(winwid, SLIDE_RAND, 1);
 	}
 	else if (feh_is_kp(&keys.toggle_caption, keysym, state)) {
 		if (opt.caption_path)
@@ -609,10 +655,16 @@ void feh_event_handle_keypress(XEvent * ev)
 		winwidget_destroy(winwid);
 	}
 	else if (feh_is_kp(&keys.orient_1, keysym, state)) {
-		feh_edit_inplace_orient(winwid, 1);
+		feh_edit_inplace(winwid, 1);
 	}
 	else if (feh_is_kp(&keys.orient_3, keysym, state)) {
-		feh_edit_inplace_orient(winwid, 3);
+		feh_edit_inplace(winwid, 3);
+	}
+	else if (feh_is_kp(&keys.flip, keysym, state)) {
+		feh_edit_inplace(winwid, INPLACE_EDIT_FLIP);
+	}
+	else if (feh_is_kp(&keys.mirror, keysym, state)) {
+		feh_edit_inplace(winwid, INPLACE_EDIT_MIRROR);
 	}
 	else if (feh_is_kp(&keys.toggle_fullscreen, keysym, state)) {
 #ifdef HAVE_LIBXINERAMA
