@@ -1,6 +1,7 @@
 /* menu.c
 
 Copyright (C) 1999-2003 Tom Gilbert.
+Copyright (C) 2010-2011 Daniel Friesel.
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to
@@ -24,8 +25,8 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
 
 #include "feh.h"
-#include "support.h"
 #include "thumbnail.h"
+#include "wallpaper.h"
 #include "winwidget.h"
 #include "filelist.h"
 #include "options.h"
@@ -34,7 +35,6 @@ Window menu_cover = 0;
 feh_menu *menu_root = NULL;
 feh_menu *menu_main = NULL;
 feh_menu *menu_single_win = NULL;
-feh_menu *menu_about_win = NULL;
 feh_menu *menu_thumbnail_viewer = NULL;
 feh_menu *menu_thumbnail_win = NULL;
 feh_menu *menu_bg = NULL;
@@ -49,7 +49,7 @@ void feh_menu_cb(feh_menu * m, feh_menu_item * i, int action, void *data);
 void feh_menu_cb_opt_fullscreen(feh_menu * m, feh_menu_item * i);
 
 enum {
-	CB_ABOUT = 1, CB_CLOSE, CB_EXIT, CB_RELOAD, CB_REMOVE, CB_DELETE, CB_RESET,
+	CB_CLOSE = 1, CB_EXIT, CB_RELOAD, CB_REMOVE, CB_DELETE, CB_RESET,
 	CB_REMOVE_THUMB, CB_DELETE_THUMB, CB_BG_TILED, CB_BG_SCALED,
 	CB_BG_CENTERED, CB_BG_FILLED, CB_BG_TILED_NOFILE,
 	CB_BG_SCALED_NOFILE, CB_BG_CENTERED_NOFILE, CB_BG_FILLED_NOFILE,
@@ -439,14 +439,12 @@ void feh_menu_show_at_submenu(feh_menu * m, feh_menu * parent_m, feh_menu_item *
 
 void feh_menu_move(feh_menu * m, int x, int y)
 {
-	int dx, dy;
-
 	if (!m)
 		return;
-	dx = x - m->x;
-	dy = y - m->y;
+
 	if (m->visible)
 		XMoveWindow(disp, m->win, x, y);
+
 	m->x = x;
 	m->y = y;
 	return;
@@ -566,7 +564,7 @@ feh_menu_item *feh_menu_add_entry(feh_menu * m, char *text, Imlib_Image icon,
 	return(mi);
 }
 
-void feh_menu_entry_get_size(feh_menu * m, feh_menu_item * i, int *w, int *h)
+void feh_menu_entry_get_size(feh_menu_item * i, int *w, int *h)
 {
 	int tw, th;
 
@@ -580,7 +578,6 @@ void feh_menu_entry_get_size(feh_menu * m, feh_menu_item * i, int *w, int *h)
 	}
 
 	return;
-	m = NULL;
 }
 
 void feh_menu_calc_size(feh_menu * m)
@@ -597,7 +594,7 @@ void feh_menu_calc_size(feh_menu * m)
 	for (i = m->items; i; i = i->next) {
 		int w, h;
 
-		feh_menu_entry_get_size(m, i, &w, &h);
+		feh_menu_entry_get_size(i, &w, &h);
 		if (w > max_w)
 			max_w = w;
 		if (h > max_h)
@@ -621,16 +618,14 @@ void feh_menu_calc_size(feh_menu * m)
 
 			im = i->icon;
 			if (im) {
-				int iw, ih, ow, oh;
+				int iw, ih, ow;
 
 				iw = gib_imlib_image_get_width(im);
 				ih = gib_imlib_image_get_height(im);
 				if (ih <= max_h) {
 					ow = iw;
-					oh = ih;
 				} else {
 					ow = (iw * max_h) / ih;
-					oh = max_h;
 				}
 				if (ow > icon_w)
 					icon_w = ow;
@@ -954,8 +949,6 @@ void feh_menu_init_main(void)
 	mi = feh_menu_add_entry(menu_main, "Options", NULL, "OPTIONS", 0, NULL, NULL);
 	mi->func_gen_sub = feh_menu_func_gen_options;
 
-	if (!opt.full_screen)
-		feh_menu_add_entry(menu_main, "About " PACKAGE, NULL, NULL, CB_ABOUT, NULL, NULL);
 	if (opt.multiwindow)
 		feh_menu_add_entry(menu_main, "Close", NULL, NULL, CB_CLOSE, NULL, NULL);
 	feh_menu_add_entry(menu_main, "Exit", NULL, NULL, CB_EXIT, NULL, NULL);
@@ -1103,17 +1096,6 @@ void feh_menu_init_common()
 	return;
 }
 
-void feh_menu_init_about_win(void)
-{
-	menu_about_win = feh_menu_new();
-	menu_about_win->name = estrdup("ABOUTWIN");
-
-	feh_menu_add_entry(menu_about_win, "Close", NULL, NULL, CB_CLOSE, NULL, NULL);
-	feh_menu_add_entry(menu_about_win, "Exit", NULL, NULL, CB_EXIT, NULL, NULL);
-
-	return;
-}
-
 void feh_menu_init_single_win(void)
 {
 	feh_menu *m;
@@ -1146,8 +1128,6 @@ void feh_menu_init_single_win(void)
 	feh_menu_add_entry(menu_single_win, NULL, NULL, NULL, 0, NULL, NULL);
 	mi = feh_menu_add_entry(menu_single_win, "Options", NULL, "OPTIONS", 0, NULL, NULL);
 	mi->func_gen_sub = feh_menu_func_gen_options;
-	feh_menu_add_entry(menu_single_win, "About " PACKAGE, NULL, NULL,
-			CB_ABOUT, NULL, NULL);
 	feh_menu_add_entry(menu_single_win, "Close", NULL, NULL, CB_CLOSE, NULL, NULL);
 	feh_menu_add_entry(menu_single_win, "Exit", NULL, NULL, CB_EXIT, NULL, NULL);
 
@@ -1176,8 +1156,6 @@ void feh_menu_init_thumbnail_win(void)
 	feh_menu_add_entry(menu_thumbnail_win, NULL, NULL, NULL, 0, NULL, NULL);
 	mi = feh_menu_add_entry(menu_thumbnail_win, "Options", NULL, "OPTIONS", 0, NULL, NULL);
 	mi->func_gen_sub = feh_menu_func_gen_options;
-	feh_menu_add_entry(menu_thumbnail_win, "About " PACKAGE, NULL, NULL,
-			CB_ABOUT, NULL, NULL);
 	feh_menu_add_entry(menu_thumbnail_win, "Close", NULL, NULL, CB_CLOSE, NULL, NULL);
 	feh_menu_add_entry(menu_thumbnail_win, "Exit", NULL, NULL, CB_EXIT, NULL, NULL);
 	return;
@@ -1215,8 +1193,6 @@ void feh_menu_init_thumbnail_viewer(void)
 	mi = feh_menu_add_entry(menu_thumbnail_viewer, "Options", NULL,
 			"OPTIONS", 0, NULL, NULL);
 	mi->func_gen_sub = feh_menu_func_gen_options;
-	feh_menu_add_entry(menu_thumbnail_viewer, "About " PACKAGE, NULL, NULL,
-			CB_ABOUT, NULL, NULL);
 	feh_menu_add_entry(menu_thumbnail_viewer, "Close", NULL, NULL, CB_CLOSE, NULL, NULL);
 	feh_menu_add_entry(menu_thumbnail_viewer, "Exit", NULL, NULL, CB_EXIT, NULL, NULL);
 	m = feh_menu_new();
@@ -1240,15 +1216,8 @@ void feh_menu_cb_opt_fullscreen(feh_menu * m, feh_menu_item * i)
 		int i, rect[4];
 
 		winwidget_get_geometry(m->fehwin, rect);
-		/* printf("window: (%d, %d)\n", rect[0], rect[1]);
-		   printf("found %d screens.\n", num_xinerama_screens); */
 		for (i = 0; i < num_xinerama_screens; i++) {
 			xinerama_screen = 0;
-			/* printf("%d: [%d, %d, %d, %d] (%d, %d)\n",
-			   i,
-			   xinerama_screens[i].x_org, xinerama_screens[i].y_org,
-			   xinerama_screens[i].width, xinerama_screens[i].height,
-			   rect[0], rect[1]); */
 			if (XY_IN_RECT(rect[0], rect[1],
 				       xinerama_screens[i].x_org,
 				       xinerama_screens[i].y_org,
@@ -1267,7 +1236,7 @@ void feh_menu_cb_opt_fullscreen(feh_menu * m, feh_menu_item * i)
 	winwidget_destroy_xwin(m->fehwin);
 	winwidget_create_window(m->fehwin, m->fehwin->im_w, m->fehwin->im_h);
 
-	winwidget_render_image(m->fehwin, 1, 1);
+	winwidget_render_image(m->fehwin, 1, 0);
 	winwidget_show(m->fehwin);
 
 #ifdef HAVE_LIBXINERAMA
@@ -1283,8 +1252,6 @@ void feh_menu_cb_opt_fullscreen(feh_menu * m, feh_menu_item * i)
 void feh_menu_cb(feh_menu * m, feh_menu_item * i, int action, void *data)
 {
 	char *path;
-	Imlib_Image im;
-	winwidget winwid;
 
 	switch (action) {
 		case CB_BG_TILED:
@@ -1319,16 +1286,6 @@ void feh_menu_cb(feh_menu * m, feh_menu_item * i, int action, void *data)
 		case CB_BG_FILLED_NOFILE:
 			feh_wm_set_bg(NULL, m->fehwin->im, 0, 0, 1, (int) data, 1);
 			break;
-		case CB_ABOUT:
-			if (feh_load_image_char(&im, PREFIX "/share/feh/images/about.png")
-					!= 0) {
-				winwid = winwidget_create_from_image(im, "About " PACKAGE,
-						WIN_TYPE_ABOUT);
-				winwid->file = gib_list_add_front(NULL,
-						feh_file_new(PREFIX "/share/feh/images/about.png"));
-				winwidget_show(winwid);
-			}
-			break;
 		case CB_CLOSE:
 			winwidget_destroy(m->fehwin);
 			break;
@@ -1342,10 +1299,10 @@ void feh_menu_cb(feh_menu * m, feh_menu_item * i, int action, void *data)
 				winwidget_resize(m->fehwin, m->fehwin->im_w, m->fehwin->im_h);
 			}
 			winwidget_reset_image(m->fehwin);
-			winwidget_render_image(m->fehwin, 1, 1);
+			winwidget_render_image(m->fehwin, 1, 0);
 			break;
 		case CB_RELOAD:
-			feh_reload_image(m->fehwin, 0, 0);
+			feh_reload_image(m->fehwin, 0, 1);
 			break;
 		case CB_REMOVE:
 			feh_filelist_image_remove(m->fehwin, 0);
@@ -1364,32 +1321,32 @@ void feh_menu_cb(feh_menu * m, feh_menu_item * i, int action, void *data)
 		case CB_SORT_FILENAME:
 			filelist = gib_list_sort(filelist, feh_cmp_filename);
 			if (opt.jump_on_resort) {
-				slideshow_change_image(m->fehwin, SLIDE_FIRST);
+				slideshow_change_image(m->fehwin, SLIDE_FIRST, 1);
 			}
 			break;
 		case CB_SORT_IMAGENAME:
 			filelist = gib_list_sort(filelist, feh_cmp_name);
 			if (opt.jump_on_resort) {
-				slideshow_change_image(m->fehwin, SLIDE_FIRST);
+				slideshow_change_image(m->fehwin, SLIDE_FIRST, 1);
 			}
 			break;
 		case CB_SORT_FILESIZE:
 			filelist = gib_list_sort(filelist, feh_cmp_size);
 			if (opt.jump_on_resort) {
-				slideshow_change_image(m->fehwin, SLIDE_FIRST);
+				slideshow_change_image(m->fehwin, SLIDE_FIRST, 1);
 			}
 			break;
 		case CB_SORT_RANDOMIZE:
 			filelist = gib_list_randomize(filelist);
 			if (opt.jump_on_resort) {
-				slideshow_change_image(m->fehwin, SLIDE_FIRST);
+				slideshow_change_image(m->fehwin, SLIDE_FIRST, 1);
 			}
 			break;
 		case CB_FIT:
 			winwidget_size_to_image(m->fehwin);
 			break;
 		case CB_EDIT_ROTATE:
-			feh_edit_inplace_orient(m->fehwin, (int) data);
+			feh_edit_inplace(m->fehwin, (int) data);
 			break;
 		case CB_SAVE_IMAGE:
 			slideshow_save_image(m->fehwin);
@@ -1403,7 +1360,7 @@ void feh_menu_cb(feh_menu * m, feh_menu_item * i, int action, void *data)
 				opt.draw_filename = TRUE;
 			else
 				opt.draw_filename = FALSE;
-			winwidget_rerender_all(0, 1);
+			winwidget_rerender_all(0);
 			break;
 		case CB_OPT_DRAW_ACTIONS:
 			MENU_ITEM_TOGGLE(i);
@@ -1411,7 +1368,7 @@ void feh_menu_cb(feh_menu * m, feh_menu_item * i, int action, void *data)
 				opt.draw_actions = TRUE;
 			else
 				opt.draw_actions = FALSE;
-			winwidget_rerender_all(0, 1);
+			winwidget_rerender_all(0);
 			break;
 		case CB_OPT_KEEP_HTTP:
 			MENU_ITEM_TOGGLE(i);
@@ -1435,8 +1392,11 @@ void feh_menu_cb(feh_menu * m, feh_menu_item * i, int action, void *data)
 			break;
 		case CB_OPT_AUTO_ZOOM:
 			MENU_ITEM_TOGGLE(i);
-			opt.auto_zoom = MENU_ITEM_IS_ON(i) ? 1 : 0;
-			winwidget_rerender_all(1, 1);
+			if (MENU_ITEM_IS_ON(i))
+				opt.zoom_mode = ZOOM_MODE_MAX;
+			else
+				opt.zoom_mode = 0;
+			winwidget_rerender_all(1);
 			break;
 	}
 	return;
@@ -1487,7 +1447,8 @@ static feh_menu *feh_menu_func_gen_options(feh_menu * m)
 	mm = feh_menu_new();
 	mm->name = estrdup("OPTIONS");
 	mm->fehwin = m->fehwin;
-	feh_menu_add_toggle_entry(mm, "Auto-Zoom", NULL, NULL, CB_OPT_AUTO_ZOOM, NULL, NULL, opt.auto_zoom);
+	feh_menu_add_toggle_entry(mm, "Auto-Zoom", NULL, NULL, CB_OPT_AUTO_ZOOM,
+				NULL, NULL, opt.zoom_mode);
 	feh_menu_add_toggle_entry(mm, "Freeze Window Size", NULL, NULL,
 				CB_OPT_FREEZE_WINDOW, NULL, NULL, opt.geom_flags);
 	feh_menu_add_toggle_entry(mm, "Fullscreen", NULL, NULL,
