@@ -31,7 +31,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 static void check_options(void);
 static void feh_getopt_theme(int argc, char **argv);
 static void feh_parse_option_array(int argc, char **argv, int finalrun);
-static void feh_check_theme_options(int arg, char **argv);
+static void feh_check_theme_options(char **argv);
 static void feh_parse_options_from_string(char *opts);
 static void feh_load_options_for_theme(char *theme);
 static void show_usage(void);
@@ -70,8 +70,6 @@ void init_parse_options(int argc, char **argv)
 	opt.prev_button = 4;
 	opt.next_button = 5;
 
-	opt.draw_actions = 0;
-
 	opt.rotate_button = 2;
 	opt.no_rotate_ctrl_mask = 0;
 	opt.blur_button = 1;
@@ -80,7 +78,6 @@ void init_parse_options(int argc, char **argv)
 	opt.start_list_at = NULL;
 	opt.jump_on_resort = 1;
 
-	opt.xinerama = 0;
 	opt.screen_clip = 1;
 #ifdef HAVE_LIBXINERAMA
 	/* if we're using xinerama, then enable it by default */
@@ -90,7 +87,7 @@ void init_parse_options(int argc, char **argv)
 	feh_getopt_theme(argc, argv);
 
 	D(("About to check for theme configuration\n"));
-	feh_check_theme_options(argc, argv);
+	feh_check_theme_options(argv);
 
 	D(("About to parse commandline options\n"));
 	/* Parse the cmdline args */
@@ -120,7 +117,7 @@ void init_parse_options(int argc, char **argv)
 	return;
 }
 
-static void feh_check_theme_options(int arg, char **argv)
+static void feh_check_theme_options(char **argv)
 {
 	if (!theme) {
 		/* This prevents screw up when running src/feh or ./feh */
@@ -137,7 +134,6 @@ static void feh_check_theme_options(int arg, char **argv)
 
 	free(theme);
 	return;
-	arg = 0;
 }
 
 static void feh_load_options_for_theme(char *theme)
@@ -363,6 +359,7 @@ static void feh_parse_option_array(int argc, char **argv, int finalrun)
 		{"no-rotate-ctrl-mask", 0, 0, 226},
 		{"no-blur-ctrl-mask", 0, 0, 227},
 		{"menu-ctrl-mask", 0, 0, 228},
+		{"draw-tinted"   , 0, 0, 229},
 
 		{"output"        , 1, 0, 'o'},
 		{"output-only"   , 1, 0, 'O'},
@@ -627,7 +624,10 @@ static void feh_parse_option_array(int argc, char **argv, int finalrun)
 			opt.alpha_level = 255 - atoi(optarg);
 			break;
 		case 'f':
-			opt.filelistfile = estrdup(optarg);
+			if (!strcmp(optarg, "-"))
+				opt.filelistfile = estrdup("/dev/stdin");
+			else
+				opt.filelistfile = estrdup(optarg);
 			break;
 		case '0':
 			opt.reload_button = atoi(optarg);
@@ -758,6 +758,9 @@ static void feh_parse_option_array(int argc, char **argv, int finalrun)
 		case 228:
 			opt.menu_ctrl_mask = 1;
 			break;
+		case 229:
+			opt.text_bg = TEXT_BG_TINTED;
+			break;
 		case 230:
 			opt.index_show_name = atoi(optarg);
 			break;
@@ -785,6 +788,8 @@ static void feh_parse_option_array(int argc, char **argv, int finalrun)
 	/* Now the leftovers, which must be files */
 	if (optind < argc) {
 		while (optind < argc) {
+			if (opt.reload)
+				original_file_items = gib_list_add_front(original_file_items, estrdup(argv[optind]));
 			/* If recursive is NOT set, but the only argument is a directory
 			   name, we grab all the files in there, but not subdirs */
 			add_file_to_filelist_recursively(argv[optind++], FILELIST_FIRST);
@@ -794,7 +799,7 @@ static void feh_parse_option_array(int argc, char **argv, int finalrun)
 		add_file_to_filelist_recursively(".", FILELIST_FIRST);
 
 	/* So that we can safely be called again */
-	optind = 1;
+	optind = 0;
 	return;
 }
 
@@ -815,15 +820,11 @@ static void check_options(void)
 	}
 
 	if (opt.full_screen && opt.multiwindow) {
-		weprintf("you shouldn't combine multiwindow mode with full-screen mode,\n"
-				"   Multiwindow mode has been disabled.");
-		opt.multiwindow = 0;
+		eprintf("You cannot combine --full-screen with --multiwindow");
 	}
 
 	if (opt.list && (opt.multiwindow || opt.index || opt.collage)) {
-		weprintf("list mode can't be combined with other processing modes,\n"
-				"   list mode disabled.");
-		opt.list = 0;
+		eprintf("You cannot combine --list with other modes");
 	}
 
 	if (opt.sort && opt.randomize) {
@@ -833,10 +834,7 @@ static void check_options(void)
 	}
 
 	if (opt.loadables && opt.unloadables) {
-		weprintf("You cant show loadables AND unloadables...\n"
-				"you might as well use ls ;)\n"
-				"loadables only will be shown\n");
-		opt.unloadables = 0;
+		eprintf("You cannot combine --loadable with --unloadable");
 	}
 
 	if (opt.thumb_title && (!opt.thumbs)) {
