@@ -83,15 +83,63 @@ void init_slideshow_mode(void)
 
 void cb_slide_timer(void *data)
 {
-	slideshow_change_image((winwidget) data, SLIDE_NEXT);
+	slideshow_change_image((winwidget) data, SLIDE_NEXT, 1);
 	return;
 }
 
 void cb_reload_timer(void *data)
 {
+	gib_list *l;
+	char *current_filename;
+
 	winwidget w = (winwidget) data;
 
-	feh_reload_image(w, 0, 0);
+	/* save the current filename for refinding it in new list */
+	current_filename = estrdup(FEH_FILE(current_file->data)->filename);
+
+	for (l = filelist; l; l = l->next) {
+		feh_file_free(l->data);
+		l->data = NULL;
+	}
+	gib_list_free_and_data(filelist);
+	filelist = NULL;
+	filelist_len = 0;
+	current_file = NULL;
+
+	/* rebuild filelist from original_file_items */
+	if (gib_list_length(original_file_items) > 0)
+		for (l = gib_list_last(original_file_items); l; l = l->prev)
+			add_file_to_filelist_recursively(l->data, FILELIST_FIRST);
+	else if (!opt.filelistfile && !opt.bgmode)
+		add_file_to_filelist_recursively(".", FILELIST_FIRST);
+	
+	if (!(filelist_len = gib_list_length(filelist))) {
+		eprintf("No files found to reload.");
+	}
+
+	/* find the previously current file */
+	for (l = filelist; l; l = l->next)
+		if (strcmp(FEH_FILE(l->data)->filename, current_filename) == 0) {
+			current_file = l;
+			break;
+		}
+
+	free(current_filename);
+
+	filelist = gib_list_first(gib_list_reverse(filelist));
+
+	if (!current_file)
+		current_file = filelist;
+	w->file = current_file;
+
+	/* reset window name in case of current file order,
+	 * filename, or filelist_length has changed.
+	 */
+	current_filename = slideshow_create_name(FEH_FILE(current_file->data));
+	winwidget_rename(w, current_filename);
+	free(current_filename);
+
+	feh_reload_image(w, 1, 0);
 	feh_add_unique_timer(cb_reload_timer, w, opt.reload);
 	return;
 }
@@ -105,6 +153,7 @@ void feh_reload_image(winwidget w, int resize, int force_new)
 
 	if (!w->file) {
 		im_weprintf(w, "couldn't reload, this image has no file associated with it.");
+		winwidget_render_image(w, 0, 0);
 		return;
 	}
 
@@ -122,20 +171,25 @@ void feh_reload_image(winwidget w, int resize, int force_new)
 	old_w = gib_imlib_image_get_width(w->im);
 	old_h = gib_imlib_image_get_height(w->im);
 
-	/* force imlib2 not to cache */
-	winwidget_free_image(w);
+	/*
+	 * If we don't free the old image before loading the new one, Imlib2's
+	 * caching will get in our way.
+	 * However, if --reload is used (force_new == 0), we want to continue if
+	 * the new image cannot be loaded, so we must not free the old image yet.
+	 */
+	if (force_new)
+		winwidget_free_image(w);
 
-	/* if the image has changed in dimensions - we gotta resize */
 	if ((feh_load_image(&tmp, FEH_FILE(w->file->data))) == 0) {
-		if (force_new) {
+		if (force_new)
 			eprintf("failed to reload image\n");
-		} else {
+		else {
 			im_weprintf(w, "Couldn't reload image. Is it still there?");
+			winwidget_render_image(w, 0, 0);
 		}
 		winwidget_rename(w, title);
 		free(title);
 		free(new_title);
-		filelist = feh_file_remove_from_list(filelist, w->file);
 		return;
 	}
 
@@ -143,6 +197,9 @@ void feh_reload_image(winwidget w, int resize, int force_new)
 			(old_h != gib_imlib_image_get_height(tmp))))
 		resize = 1;
 
+	if (!force_new)
+		winwidget_free_image(w);
+
 	w->im = tmp;
 	winwidget_reset_image(w);
 
@@ -170,9 +227,8 @@ void feh_reload_image(winwidget w, int resize, int force_new)
 	return;
 }
 
-void slideshow_change_image(winwidget winwid, int change)
+void slideshow_change_image(winwidget winwid, int change, int render)
 {
-	int success = 0;
 	gib_list *last = NULL;
 	int i = 0;
 	int jmp = 1;
@@ -253,7 +309,6 @@ void slideshow_change_image(winwidget winwid, int change)
 
 		if ((winwidget_loadimage(winwid, FEH_FILE(current_file->data)))
 		    != 0) {
-			success = 1;
 			winwid->mode = MODE_NORMAL;
 			winwid->file = current_file;
 			if ((winwid->im_w != gib_imlib_image_get_width(winwid->im))
@@ -262,7 +317,8 @@ void slideshow_change_image(winwidget winwid, int change)
 			winwidget_reset_image(winwid);
 			winwid->im_w = gib_imlib_image_get_width(winwid->im);
 			winwid->im_h = gib_imlib_image_get_height(winwid->im);
-			winwidget_render_image(winwid, 1, 0);
+			if (render)
+				winwidget_render_image(winwid, 1, 0);
 
 			s = slideshow_create_name(FEH_FILE(current_file->data));
 			winwidget_rename(winwid, s);
@@ -430,7 +486,7 @@ void feh_filelist_image_remove(winwidget winwid, char do_delete)
 		gib_list *doomed;
 
 		doomed = current_file;
-		slideshow_change_image(winwid, SLIDE_NEXT);
+		slideshow_change_image(winwid, SLIDE_NEXT, 0);
 		if (do_delete)
 			filelist = feh_file_rm_and_free(filelist, doomed);
 		else
@@ -443,6 +499,7 @@ void feh_filelist_image_remove(winwidget winwid, char do_delete)
 		s = slideshow_create_name(FEH_FILE(winwid->file->data));
 		winwidget_rename(winwid, s);
 		free(s);
+		winwidget_render_image(winwid, 1, 0);
 	} else if ((winwid->type == WIN_TYPE_SINGLE)
 		   || (winwid->type == WIN_TYPE_THUMBNAIL_VIEWER)) {
 		if (do_delete)
@@ -472,7 +529,14 @@ void slideshow_save_image(winwidget win)
 	if (!opt.quiet)
 		printf("saving image to filename '%s'\n", tmpname);
 
-	gib_imlib_save_image_with_error_return(win->im, tmpname, &err);
+	/* XXX gib_imlib_save_image_with_error_return breaks with *.XXX and
+	 * similar because it tries to set the image format, which only works
+	 * with .xxx .
+	 * So we leave that part out.
+	 */
+	imlib_context_set_image(win->im);
+	imlib_save_image_with_error_return(tmpname, &err);
+
 	if (err)
 		im_weprintf(win, "Can't save image %s:", tmpname);
 
