@@ -262,7 +262,7 @@ void winwidget_create_window(winwidget ret, int w, int h)
 		ev.xclient.display = disp;
 		ev.xclient.window = ret->win;
 		ev.xclient.format = 32;
-		ev.xclient.data.l[0] = (ret->full_screen ? 1 : 0);
+		ev.xclient.data.l[0] = 1;
 		ev.xclient.data.l[1] = prop_fs;
 
 		XChangeProperty(disp, ret->win, prop_state, XA_ATOM, 32,
@@ -350,7 +350,10 @@ void winwidget_setup_pixmaps(winwidget winwid)
 			if (winwid->gc == None) {
 				XGCValues gcval;
 
-				gcval.foreground = BlackPixel(disp, DefaultScreen(disp));
+				if (!strcmp(opt.image_bg, "white"))
+					gcval.foreground = WhitePixel(disp, DefaultScreen(disp));
+				else
+					gcval.foreground = BlackPixel(disp, DefaultScreen(disp));
 				winwid->gc = XCreateGC(disp, winwid->win, GCForeground, &gcval);
 			}
 			winwid->bg_pmap = XCreatePixmap(disp, winwid->win, scr->width, scr->height, depth);
@@ -618,8 +621,6 @@ Pixmap feh_create_checks(void)
 	Imlib_Image checks = NULL;
 
 	if (checks_pmap == None) {
-		int onoff, x, y;
-
 		checks = imlib_create_image(16, 16);
 
 		if (!checks)
@@ -630,18 +631,9 @@ Pixmap feh_create_checks(void)
 		else if (strncmp(opt.image_bg, "black", 5) == 0)
 			gib_imlib_image_fill_rectangle(checks, 0, 0, 16, 16, 0, 0, 0, 255);
 		else {
-			for (y = 0; y < 16; y += 8) {
-				onoff = (y / 8) & 0x1;
-				for (x = 0; x < 16; x += 8) {
-					if (onoff)
-						gib_imlib_image_fill_rectangle(checks, x, y, 8, 8, 144, 144, 144, 255);
-					else
-						gib_imlib_image_fill_rectangle(checks, x, y, 8, 8, 100, 100, 100, 255);
-					onoff++;
-					if (onoff == 2)
-						onoff = 0;
-				}
-			}
+			gib_imlib_image_fill_rectangle(checks, 0, 0, 16, 16, 144, 144, 144, 255);
+			gib_imlib_image_fill_rectangle(checks, 0, 0,  8,  8, 100, 100, 100, 255);
+			gib_imlib_image_fill_rectangle(checks, 8, 8,  8,  8, 100, 100, 100, 255);
 		}
 
 		checks_pmap = XCreatePixmap(disp, root, 16, 16, depth);
@@ -772,20 +764,48 @@ void winwidget_resize(winwidget winwid, int w, int h)
 	Window ignored_window;
 	XWindowAttributes attributes;
 	int tc_x, tc_y;
+	int scr_width = scr->width;
+	int scr_height = scr->height;
+
+	XGetWindowAttributes(disp, winwid->win, &attributes);
+
+#ifdef HAVE_LIBXINERAMA
+	if (opt.xinerama && xinerama_screens) {
+		int i;
+		xinerama_screen = 0;
+		for (i = 0; i < num_xinerama_screens; i++) {
+			if (XY_IN_RECT(attributes.x, attributes.y,
+						xinerama_screens[i].x_org,
+						xinerama_screens[i].y_org,
+						xinerama_screens[i].width, xinerama_screens[i].height)) {
+				xinerama_screen = i;
+				break;
+			}
+
+		}
+		if (getenv("XINERAMA_SCREEN"))
+			xinerama_screen = atoi(getenv("XINERAMA_SCREEN"));
+
+		scr_width = xinerama_screens[xinerama_screen].width;
+		scr_height = xinerama_screens[xinerama_screen].height;
+	}
+#endif
+
+
+	D(("   x %d y %d w %d h %d\n", attributes.x, attributes.y, winwid->w,
+		winwid->h));
 
 	if (opt.geom_flags) {
 		winwid->had_resize = 1;
 		return;
 	}
 	if (winwid && ((winwid->w != w) || (winwid->h != h))) {
-		D(("Really doing a resize\n"));
 		/* winwidget_clear_background(winwid); */
 		if (opt.screen_clip) {
-			winwid->w = (w > scr->width) ? scr->width : w;
-			winwid->h = (h > scr->height) ? scr->height : h;
+			winwid->w = (w > scr_width) ? scr_width : w;
+			winwid->h = (h > scr_height) ? scr_height : h;
 		}
 		if (winwid->full_screen) {
-			XGetWindowAttributes(disp, winwid->win, &attributes);
 			XTranslateCoordinates(disp, winwid->win, attributes.root,
 						-attributes.border_width -
 						attributes.x,
@@ -799,25 +819,8 @@ void winwidget_resize(winwidget winwid, int w, int h)
 		winwid->had_resize = 1;
 		XFlush(disp);
 
-#ifdef HAVE_LIBXINERAMA
-		if (opt.xinerama && xinerama_screens) {
-			int i;
-
-			for (i = 0; i < num_xinerama_screens; i++) {
-				xinerama_screen = 0;
-				if (XY_IN_RECT(winwid->x, winwid->y,
-					       xinerama_screens[i].x_org,
-					       xinerama_screens[i].y_org,
-					       xinerama_screens[i].width, xinerama_screens[i].height)) {
-					xinerama_screen = i;
-					break;
-				}
-
-			}
-			if (getenv("XINERAMA_SCREEN"))
-				xinerama_screen = atoi(getenv("XINERAMA_SCREEN"));
-		}
-#endif				/* HAVE_LIBXINERAMA */
+		D(("-> x %d y %d w %d h %d\n", winwid->x, winwid->y, winwid->w,
+			winwid->h));
 
 	} else {
 		D(("No resize actually needed\n"));
